/**
 * @file turbo_json.h
 * @brief JSON serialization for turbo results
 *//*
 * Copyright 2024-2026 by Advens and Jeremie Jourdin.
 * Copyright 2015-2026 by Rainer Gerhards and Adiscon GmbH.
 *
 * Released under ASL 2.0.
 */
#ifndef LIBLOGNORM_TURBO_JSON_H_INCLUDED
#define	LIBLOGNORM_TURBO_JSON_H_INCLUDED

#include "config.h"

#ifdef ENABLE_TURBO

#include "turbo_result_fast.h"
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

#endif /* ENABLE_TURBO */

#endif /* LIBLOGNORM_TURBO_JSON_H_INCLUDED */
